/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.carlesfigueras.passwordvault;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.BufferedBlockCipher;
import org.bouncycastle.crypto.engines.TwofishEngine;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.paddings.PaddedBufferedBlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.util.encoders.Base64;

/**
 * This class represents an account to acces to any site
 * and manage the criptographic algorithm to encode and decode
 * all data. All data is contained in a crypted form and served
 * as a plain text to the user.
 * 
 * @author Carles Figueras Garcia
 */
public class Account {
    /* Variables */
    private int id;
    private String user;
    private String password;
    private String descripcio;
    private String url;
    private BlockCipher engine = new TwofishEngine();
    
    /**
     * Constructor used to get data from database
     * 
     * @param id the database identificator
     * @param user the user of this account
     * @param password the password of this account
     * @param descripcio the description of this account
     * @param url the url of this account
     */
    public Account (int id, String user, String password, String descripcio, String url) {
        this.id = id;
        this.user = user;
        this.password = password;
        this.descripcio = descripcio;
        this.url = url;
    }

    /**
     * Constructor used to get data from database
     * 
     * @param id the database identificator
     * @param user the user of this account
     * @param password the password of this account
     * @param descripcio the description of this account
     * @param url the url of this account
     * @param key the key to encrypt/decrypt
     */
    public Account (int id, String user, String password, String descripcio, String url, String key) {
        this.id = id;
        this.user = decrypt(user, key);
        this.password = decrypt(password, key);
        this.descripcio = decrypt(descripcio, key);
        this.url = decrypt(url, key);
    }
    
    /**
     * Constructor to generate a new account
     * 
     * @param user the user of this account
     * @param password the password of this account
     * @param descripcio the description of this account
     * @param url the url of this account
     * @param key the key to encrypt/decrypt
     */
    public Account (String user, String password, String descripcio, String url, String key) {
        this.user = encrypt(user, key);
        this.password = encrypt(password, key);
        this.descripcio = encrypt(descripcio, key);
        this.url = encrypt(url, key);
    }
    
    /**
     * Constructor used to update the database
     * 
     * @param key the key to encrypt/decrypt
     * @param id the account id
     * @param user the user of this account
     * @param password the password of this account
     * @param descripcio the description of this account
     * @param url the url of this account
     */
    public Account (String key, int id, String user, String password, String descripcio, String url) {
        this.id = id;
        this.user = encrypt(user, key);
        this.password = encrypt(password, key);
        this.descripcio = encrypt(descripcio, key);
        this.url = encrypt(url, key);
    }
    
    /**
     * Encrypt plaintext data in Twofish Algorithm
     * 
     * @param data the plaintext to encrypt
     * @param clau the key needed to encrypt
     * @return the encrypted text
     */
    private String encrypt(String data, String clau) {
        String ret;
        
        if (data.isEmpty()) {
            ret = "";
        }
        else {
            byte[] key = clau.getBytes();
            byte[] dataBytes = data.getBytes();
            BufferedBlockCipher cipher = new PaddedBufferedBlockCipher(new CBCBlockCipher(engine));
            cipher.init(true, new KeyParameter(key));
            byte[] encryptedBytes = new byte[cipher.getOutputSize(dataBytes.length)];
            int tam = cipher.processBytes(dataBytes, 0, dataBytes.length, encryptedBytes, 0);
            try {
                cipher.doFinal(encryptedBytes, tam);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            ret = new String(Base64.encode(encryptedBytes)).trim();
        }
        
        return ret;
    }
    
    /**
     * Decrypt encrypted data in Twofish Algorithm
     * 
     * @param data the encrypted data to decrypt
     * @param clau the key used to encrypt this data
     * @return the plaintext
     */
    private String decrypt(String data, String clau) {
        String ret;
        
        if (data.isEmpty()) {
            ret = "";
        }
        else {
            byte[] key = clau.getBytes();
            byte[] cipherBytes = Base64.decode(data);
            BufferedBlockCipher cipher = new PaddedBufferedBlockCipher(new CBCBlockCipher(engine));
            cipher.init(false, new KeyParameter(key));
            byte[] plainBytes = new byte[cipher.getOutputSize(cipherBytes.length)];
            int tam = cipher.processBytes(cipherBytes, 0, cipherBytes.length, plainBytes, 0);
            try {
                cipher.doFinal(plainBytes, tam);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            ret = new String(plainBytes).trim();
        }
        
        return ret;
    }

    /**
     * Get the database id
     * 
     * @return the id
     */
    public int getId() {
        return id;
    }
    
    /**
     * Set the database id
     * 
     * @param id the identificator to set
     */
    public void setId(int id) {
        this.id = id;
    }
    
    /**
     * Get the user
     * 
     * @return the user
     */
    public String getUser() {
        return user;
    }
    
    /**
     * Get the user (in plain text)
     * 
     * @param key the key needed to decrypt
     * @return the decrypted user
     */
    public String getUser(String key) {
        return decrypt(user, key);
    }
    
    /**
     * Get the password
     * 
     * @return the password
     */
    public String getPassword() {
        return password;
    }
    
    /**
     * Get the password (in plain text)
     * 
     * @param key the key needed to decrypt
     * @return the decrypted password
     */
    public String getPassword(String key) {
        return decrypt(password, key);
    }
    
    /**
     * Get the description
     * 
     * @return the description
     */
    public String getDescripcio() {
        return descripcio;
    }
    
    /**
     * Get the description (in plain text)
     * 
     * @param key the key needed to decrypt
     * @return the decrypted description
     */
    public String getDescripcio(String key) {
        return decrypt(descripcio, key);
    }
    
    /**
     * Get the url
     * 
     * @return the url
     */
    public String getUrl() {
        return url;
    }
    
    /**
     * Get the url (in plain text)
     * 
     * @param key the key needed to decrypt
     * @return the decrypted url
     */
    public String getUrl(String key) {
        return decrypt(url, key);
    }
    
    /**
     * Gets the description of all account of the user
     * 
     * @param key the key needed to decrypt
     * @return The array of all decrypted descriptions
     */
    public Object[] getObjectDetails(String key) {
        Object[] object = new Object[2];
        
        object[0] = this.id;
        object[1] = getDescripcio(key);
        
        return object;
    }
   
}
